"""
BREACH.AI v2 - Proof Generation Modules

1 MVP Proof Module:
1. evidence_generator - Generate undeniable breach evidence
"""

from breach.modules.proof.evidence_generator import EvidenceGenerator

__all__ = [
    "EvidenceGenerator",
]
